@@ -91,6 +91,11 @@ public class SVNOption extends AbstractSVNOption {
     public static final SVNOption GIT_DIFF_FORMAT = new SVNOption("git");
     public static final SVNOption DIFF = new SVNOption("diff");
     public static final SVNOption SHOW_INHERITED_PROPS = new SVNOption("show-inherited-props");
+    public static final SVNOption SHOW_ITEM = new SVNOption("show-item", false);
+    public static final SVNOption NO_NEWLINE = new SVNOption("no-newline");
+    public static final SVNOption PIN_EXTERNALS = new SVNOption("pin-externals");
+    public static final SVNOption REMOVE_UNVERSIONED = new SVNOption("remove-unversioned");
+    public static final SVNOption REMOVE_IGNORED = new SVNOption("remove-ignored");
 
     // auth options.
     public static final SVNOption USERNAME = new SVNOption("username", false);
@@ -99,6 +104,7 @@ public class SVNOption extends AbstractSVNOption {
     public static final SVNOption NON_INTERACTIVE = new SVNOption("non-interactive");
     public static final SVNOption FORCE_INTERACTIVE = new SVNOption("force-interactive");
     public static final SVNOption TRUST_SERVER_CERT = new SVNOption("trust-server-cert");
+    public static final SVNOption TRUST_SERVER_CERT_FAILURES = new SVNOption("trust-server-cert-failures", false);
 
     public static final SVNOption STRIP = new SVNOption("strip", "p", false);
     public static final SVNOption ALLOW_MIXED_REVISIONS = new SVNOption("allow-mixed-revisions");
